#include <iostream.h>
#include <fstream.h>
#include <string>
#include <ctime>	// for random numbers

#include "includes/parameters.c"
#include "includes/waveform.c"
#include "includes/hadamard.c"
#include "includes/haar.c"
#include "includes/faults.c"

#define N 16

void print_array (float * array) {
  int i;
  for (i=0; i<N; i++) {
    cout << array[i] << ", ";
  }
  cout << "\n";
}

void program_description(Parameters &p_in) {
  // prints out a description of the program
  cout << "-------------------------\n";
  cout << " ** Waveform Generator **\n\n";
  cout << " - generates random waveform\n";
  cout << " Requires:\n";
  cout << " - waveformfileout\n";
  cout << " - waveformlength\n";
  cout << " - numprimaryinputs\n";
  cout << "-------------------------\n\n";

  // set param flag:
  p_in.set_flag(PARAM_WAVEFORMFILEOUT | PARAM_WAVEFORMLENGTH | PARAM_NUMPRIMARYINPUTS);
}

int main(int argc, char *argv[]) {

  Parameters param;
  program_description(param);

  int i;

  // initialize random numbers 
  srand(static_cast<unsigned>(time(0))); 

  if (argc > 1) {
    param.load(argv[1]);
  } else {
    param.load();
  }
  
  Waveform * w;
  w = new Waveform;

  w->rand_wave_new(param.waveformlength, param.numprimaryinputs);
  w->save(param.waveformfileout);

  cout << "1) random waveform of length " << param.waveformlength << " with ";
  cout << param.numprimaryinputs << " inputs created\n";
  cout << "2) saved to file: " << param.waveformfileout << "\n\n";

  delete w;
  return 0;
}
