#include "faults.h"
#include "faultitem.c"

#ifndef faults_c
#define faults_c

Faults::Faults () {
  top = NULL;
}

Faults::~Faults () {
  delete top; // this should delete all elements of the list
  top = NULL;
}

int Faults::num_faults () {
  Faultitem * current;  // local for pointing & moving
  int i=0;
  current = top;
  while (current != NULL) {
    i++;
    current = current->next;
  }
  return i;
}

void Faults::print () {
  Faultitem * current;  // local for pointing & moving
  int i=0;
  current = top;
  while (current != NULL) {
    i++;
    cout << i << ": @ " << current->fault_time << ", #" << current->fault_id << "\n";
    current = current->next;
  }
}

int Faults::load(string file_name) {
  char var_fault_id[10], var_time[10];
  bool keep_parsing=true;
  // load in the faults
  fstream file;
  file.open(file_name.c_str(), ios::in);
  while (!file.eof () && keep_parsing) {
    file.getline(var_fault_id, 10, ' ');
    if (((string) var_fault_id).length() > 0) {
      file.getline(var_time, 10, '\n');
      if (((string) var_time).length() > 0) {
	keep_parsing = insert_fault(atoi(var_time), atoi(var_fault_id));
      } else {
	keep_parsing = false;
      }
    } else {
      keep_parsing = false;
    }
  }
  file.close();

  // return the number of faults
  return num_faults();
}

bool Faults::insert_fault(int fault_time, int fault_id) {
  Faultitem * current, * previous;  // local for pointing & moving
  bool insert_here = false;

  current = top;
  if (current == NULL) {
    // list is empty, insert:
    top = new Faultitem(fault_time, fault_id);
  } else { // add to list:
    insert_here = false;
    // check to see if goes at top of list:
    if (fault_time < current->fault_time) {
      insert_here = true;
    } else if (fault_time == current->fault_time && fault_id < current->fault_id) {
      insert_here = true;
    }
    if (insert_here) { // insert at top
      // insert before
      top = new Faultitem(fault_time, fault_id);
      top->next = current; // point forward
      current->prev = top; // point back
    } else { // goes in middle (or end) of list
      if (current->next == NULL) { // only one element in list, so add to end
	current->next = new Faultitem(fault_time, fault_id);
	current->next->prev = current;  // point back
      } else { // multi-element list, iterate through
	current = current->next;
	while (current != NULL) {
	  previous = current->prev; // pointer to previous element
	  insert_here = false;
	  // check to see if insert inbetween previous & current:
	  if (fault_time > previous->fault_time) {
	    if (fault_time < current->fault_time) {
	      insert_here = true;
	    } else if (fault_time == current->fault_time && fault_id < current->fault_id) {
	      insert_here = true;
	    }
	  } else if (fault_time == previous->fault_time && fault_id > previous->fault_id) {
	    if (fault_time == current->fault_time && fault_id < current->fault_id) {
	      insert_here = true;
	    } else if (fault_time < current->fault_time) {
	      insert_here = true;
	    }
	  }
	  if (insert_here) {
	    previous->next = new Faultitem(fault_time, fault_id);
	    previous->next->prev = previous; // point new back
	    previous->next->next = current; // point new forward
	    current->prev = previous->next; // point current one back to new
	    current = NULL; // break out of while loop
	  } else { // not inserting here
	    // check to see if at end of the list
	    // if so, need to add to end of the list
	    if (current->next == NULL) {
	      current->next = new Faultitem(fault_time, fault_id);
	      current->next->prev = current;  // point back
	      current = NULL; // break out of while loop
	    } else { // move on
	      current = current->next;
	    }
	  }
	} // end while loop
      }
    }
  }
}

#endif
