#include "parameters.h"

#ifndef parameters_c
#define parameters_c

Parameters::Parameters () {
  ask_for_param_file = false;
  flag = 0;
  paramfile = "parameters.txt";

  autoload = false;
  ommitvector = false;
  waveformfilein = "";
  waveformfileout = "";
  waveformstorage = "";
  waveformlength = 0;
  numprimaryinputs = 0;
  transformtype = "";
  transformsize = 0;
  dothreshold = false;
  faultfile = "";
}

Parameters::~Parameters () {
}

void Parameters::clear_flag() {
  flag = 0;
}

void Parameters::set_flag (int flag_in) {
  flag |= flag_in;
}

void Parameters::load() {
  ask_for_param_file = true;
  local_load();
}

void Parameters::load(string paramfilein) {
  if (paramfilein == "none") {
    paramfile = "";
  } else {
    paramfile = paramfilein;
  }
  ask_for_param_file = false;
  local_load();
}

void Parameters::local_load() {
  int i;
  string temp_string;
  char temp_char[PARAMETER_LINE_LENGTH];

  // don't ignore whitespace
  cin.unsetf(ios::skipws);

  if (ask_for_param_file) {
    cout << "Parameters file ('none' for none) [" << paramfile << "]: ";
    cin >> temp_string;
    if (temp_string.length() > 0 && temp_string != "none") {
      paramfile = temp_string;
    } else if (temp_string == "none") {
      paramfile = "";
    }
    cin.clear(); cin.ignore(); temp_string.erase();
  }

  load_parameters();

  if (!autoload) {

    if (flag & PARAM_OMMITVECTOR) {
      cout << "Waveform has ommit vector: [" << (ommitvector?"true":"false") << "]: ";
      cin >> temp_string;
      if (temp_string.length() > 0) {
	ommitvector = (temp_string.at(0) == 't') ? true : false;
      }
      cin.clear(); cin.ignore(); temp_string.erase();
    }
    
    if (flag & PARAM_WAVEFORMFILEIN) {
      cout << "Waveform file in: [" << waveformfilein << "]: ";
      cin >> temp_string;
      if (temp_string.length() > 0) {
	waveformfilein = temp_string;
      }
      cin.clear(); cin.ignore(); temp_string.erase();
    }

    if (flag & PARAM_WAVEFORMFILEOUT) {
      cout << "Waveform file out: [" << waveformfileout << "]: ";
      cin >> temp_string;
      if (temp_string.length() > 0) {
	waveformfileout = temp_string;
      }
      cin.clear(); cin.ignore(); temp_string.erase();
    }

    if (flag & PARAM_WAVEFORMSTORAGE) {
      cout << "Waveform storage file: [" << waveformstorage << "]: ";
      cin >> temp_string;
      if (temp_string.length() > 0) {
	waveformstorage = temp_string;
      }
      cin.clear(); cin.ignore(); temp_string.erase();
    }

    if (flag & PARAM_WAVEFORMLENGTH) {
      cout << "Waveform length: [" << waveformlength << "]: ";
      cin >> temp_string;
      if (temp_string.length() > 0) {
	waveformlength = atoi(temp_string.c_str());
      }
      cin.clear(); cin.ignore(); temp_string.erase();
    }

    if (flag & PARAM_NUMPRIMARYINPUTS) {
      cout << "Number of primary inputs: [" << numprimaryinputs << "]: ";
      cin >> temp_string;
      if (temp_string.length() > 0) {
	numprimaryinputs = atoi(temp_string.c_str());
      }
      cin.clear(); cin.ignore(); temp_string.erase();
    }

    if (flag & PARAM_TRANSFORMTYPE) {
      cout << "Type of transform: [" << transformtype << "]: ";
      cin >> temp_string;
      if (temp_string.length() > 0) {
	transformtype = temp_string;
      }
      cin.clear(); cin.ignore(); temp_string.erase();
    }

    if (flag & PARAM_TRANSFORMSIZE) {
      cout << "Size of transform: [" << transformsize << "]: ";
      cin >> temp_string;
      if (temp_string.length() > 0) {
	transformsize = atoi(temp_string.c_str());
      }
      cin.clear(); cin.ignore(); temp_string.erase();
    }

    if (flag & PARAM_DOTHRESHOLD) {
      cout << "Do threshold: [" << (dothreshold?"true":"false") << "]: ";
      cin >> temp_string;
      if (temp_string.length() > 0) {
	dothreshold = (temp_string.at(0) == 't') ? true : false;
      }
      cin.clear(); cin.ignore(); temp_string.erase();
    }

    if (flag & PARAM_FAULTFILE) {
      cout << "Fault/Time file: [" << faultfile << "]: ";
      cin >> temp_string;
      if (temp_string.length() > 0) {
	faultfile = temp_string;
      }
      cin.clear(); cin.ignore(); temp_string.erase();
    }

    // end "if not autoload"
  }

  cout << "\n";
}

void Parameters::load_parameters() {
  char var_name[PARAMETER_LINE_LENGTH], var_value[PARAMETER_LINE_LENGTH];
  char peek_val;
  bool keep_parsing = true;
  if (paramfile.length() > 1) {
    fstream file;
    file.open(paramfile.c_str(), ios::in);
    while (!file.eof () && keep_parsing) {
      peek_val = file.peek();
      if (peek_val == '#' || peek_val == ' ' || peek_val == '\n') {
	file.getline(var_value, PARAMETER_LINE_LENGTH, '\n');
      } else {
	file.getline(var_name, PARAMETER_LINE_LENGTH, '=');
	file.getline(var_value, PARAMETER_LINE_LENGTH, '\n');
	keep_parsing = parse_parameters((string) var_name, (string) var_value);
      }
    }
    file.close();
    cout << "Loaded parameters from file: " << paramfile.c_str() << "\n";
  }
}

bool Parameters::parse_parameters(string name, string value) {
  if (name.length() > 1) {
    if (name == "autoload") {
      autoload = (value.at(0) == 't') ? true : false;
    } else if (name == "ommitvector") {
      ommitvector = (value.at(0) == 't') ? true : false;
    } else if (name == "waveformfilein") {
      waveformfilein = value;
    } else if (name == "waveformfileout") {
      waveformfileout = value;
    } else if (name == "waveformstorage") {
      waveformstorage = value;
    } else if (name == "waveformlength") {
      waveformlength = atoi(value.c_str());
    } else if (name == "numprimaryinputs") {
      numprimaryinputs = atoi(value.c_str());
    } else if (name == "transformtype") {
      transformtype = value;
    } else if (name == "transformsize") {
      transformsize = atoi(value.c_str());
    } else if (name == "dothreshold") {
      dothreshold = (value.at(0) == 't') ? true : false;
    } else if (name == "faultfile") {
      faultfile = value;
    } else {
      cout << "[don't recognize parameter: " << name << "]\n";
      //      return false;
    }
    return true;
  } else {
    return true;
  }
}

#endif
