#include "transform.h"

#ifndef transform_c
#define transform_c

Transform::Transform (int new_size) {
  threshold_value = 0; // initialize
  p = 0;
  size = (int) check_size(new_size, new_size);
  if (size > 0) {
    data = new float * [size];
    int i, j;
    for (i=0; i<size; i++) {
      data[i] = new float[size];
      for (j=0; j<size; j++) {
	data[i][j] = 0;
      }
    }
  } else {
    cout << "Could not create matrix. (bad size: " << new_size << ")\n";
    size = 0;
    p = 0;
  }
}

Transform::~Transform () {
  int i;
  if (size > 0) {
    for (i=0; i<size; i++) {
      delete [] data[i];
    }
    delete [] data;
  }
}

float Transform::check_size (float new_size, int orig) {
  if (new_size >= 1.0) {
    if (new_size == 1) {
      return orig;
    } else {
      p++;
      return check_size(new_size/2.0, orig);
    }
  } else {
    return 0;
  }
}

// absolute value function
float Transform::Abs(float Nbr)
{
  return (Nbr >= 0) ? Nbr : -Nbr;
}

float Transform::value (int x, int y) {
  if (x < 0 || x >= size || y < 0 || y >= size || size == 0) {
    return 0.0;
  } else {
    return data[x][y];
  }
}

void Transform::print () {
  int i,j;
  for (i=0; i<size; i++) {
    for (j=0; j<size; j++) {
      if (value(i,j) < 0) {
	cout << "[" << value(i,j) << " ]";
      } else {
	cout << "[ " << value(i,j) << " ]";
      }
    }
    cout << "\n";
  }
}

void Transform::set_threshold (int new_thresh) {
  threshold_value = new_thresh;
}

void Transform::transform (float * new_array, float * out_array) {
  int i,j;
  for (i=0; i<size; i++) {
    out_array[i] = 0;
  }
  for (i=0; i<size; i++) {
    for (j=0; j<size; j++) {
      out_array[i] += value(i,j)*new_array[j];
    }
  }
}

void Transform::itransform (float * new_array, float * out_array) {
  int i,j;
  for (i=0; i<size; i++) {
    out_array[i] = 0;
  }
  for (i=0; i<size; i++) {
    for (j=0; j<size; j++) {
      out_array[i] += value(j,i)*new_array[j];
    }
  }
  for (i=0; i<size; i++) {
    out_array[i] = out_array[i]/(size);
  }
}

#endif
